@@ -26,7 +26,7 @@
 #include "arch.h"
 #include "arch-aarch64.h"
 
-/* NOTE: based on Linux 3.19 */
+/* NOTE: based on Linux 4.10-rc6+ */
 const struct arch_syscall_def aarch64_syscall_table[] = { \
 	{ "_llseek", __PNR__llseek },
 	{ "_newselect", __PNR__newselect },
@@ -65,6 +65,7 @@ const struct arch_syscall_def aarch64_syscall_table[] = { \
 	{ "clone", 220 },
 	{ "close", 57 },
 	{ "connect", 203 },
+	{ "copy_file_range", 285 },
 	{ "creat", __PNR_creat },
 	{ "create_module", __PNR_create_module },
 	{ "delete_module", 106 },
@@ -196,6 +197,7 @@ const struct arch_syscall_def aarch64_syscall_table[] = { \
 	{ "lstat64", __PNR_lstat64 },
 	{ "madvise", 233 },
 	{ "mbind", 235 },
+	{ "membarrier", 283 },
 	{ "memfd_create", 279 },
 	{ "migrate_pages", 238 },
 	{ "mincore", 232 },
@@ -204,6 +206,7 @@ const struct arch_syscall_def aarch64_syscall_table[] = { \
 	{ "mknod", __PNR_mknod },
 	{ "mknodat", 33 },
 	{ "mlock", 228 },
+	{ "mlock2", 284 },
 	{ "mlockall", 230 },
 	{ "mmap", 222 },
 	{ "mmap2", __PNR_mmap2 },
@@ -224,6 +227,7 @@ const struct arch_syscall_def aarch64_syscall_table[] = { \
 	{ "msgrcv", 188 },
 	{ "msgsnd", 189 },
 	{ "msync", 227 },
+	{ "multiplexer", __PNR_multiplexer },
 	{ "munlock", 229 },
 	{ "munlockall", 231 },
 	{ "munmap", 215 },
@@ -250,11 +254,15 @@ const struct arch_syscall_def aarch64_syscall_table[] = { \
 	{ "pipe", __PNR_pipe },
 	{ "pipe2", 59 },
 	{ "pivot_root", 41 },
+	{ "pkey_alloc", 289 },
+	{ "pkey_free", 290 },
+	{ "pkey_mprotect", 288 },
 	{ "poll", __PNR_poll },
 	{ "ppoll", 73 },
 	{ "prctl", 167 },
 	{ "pread64", 67 },
 	{ "preadv", 69 },
+	{ "preadv2", 286 },
 	{ "prlimit64", 261 },
 	{ "process_vm_readv", 270 },
 	{ "process_vm_writev", 271 },
@@ -265,6 +273,7 @@ const struct arch_syscall_def aarch64_syscall_table[] = { \
 	{ "putpmsg", __PNR_putpmsg },
 	{ "pwrite64", 68 },
 	{ "pwritev", 70 },
+	{ "pwritev2", 287 },
 	{ "query_module", __PNR_query_module },
 	{ "quotactl", 60 },
 	{ "read", 63 },
@@ -294,6 +303,10 @@ const struct arch_syscall_def aarch64_syscall_table[] = { \
 	{ "rt_sigsuspend", 133 },
 	{ "rt_sigtimedwait", 137 },
 	{ "rt_tgsigqueueinfo", 240 },
+	{ "rtas", __PNR_rtas },
+	{ "s390_pci_mmio_read", __PNR_s390_pci_mmio_read },
+	{ "s390_pci_mmio_write", __PNR_s390_pci_mmio_write },
+	{ "s390_runtime_instr", __PNR_s390_runtime_instr },
 	{ "sched_get_priority_max", 125 },
 	{ "sched_get_priority_min", 126 },
 	{ "sched_getaffinity", 123 },
@@ -372,6 +385,8 @@ const struct arch_syscall_def aarch64_syscall_table[] = { \
 	{ "socketcall", __PNR_socketcall },
 	{ "socketpair", 199 },
 	{ "splice", 76 },
+	{ "spu_create", __PNR_spu_create },
+	{ "spu_run", __PNR_spu_run },
 	{ "ssetmask", __PNR_ssetmask },
 	{ "stat", __PNR_stat },
 	{ "stat64", __PNR_stat64 },
@@ -379,8 +394,11 @@ const struct arch_syscall_def aarch64_syscall_table[] = { \
 	{ "statfs64", __PNR_statfs64 },
 	{ "stime", __PNR_stime },
 	{ "stty", __PNR_stty },
+	{ "subpage_prot", __PNR_subpage_prot },
+	{ "swapcontext", __PNR_swapcontext },
 	{ "swapoff", 225 },
 	{ "swapon", 224 },
+	{ "switch_endian", __PNR_switch_endian },
 	{ "symlink", __PNR_symlink },
 	{ "symlinkat", 36 },
 	{ "sync", 81 },
@@ -388,6 +406,7 @@ const struct arch_syscall_def aarch64_syscall_table[] = { \
 	{ "sync_file_range2", __PNR_sync_file_range2 },
 	{ "syncfs", 267 },
 	{ "syscall", __PNR_syscall },
+	{ "sys_debug_setcontext", __PNR_sys_debug_setcontext },
 	{ "sysfs", __PNR_sysfs },
 	{ "sysinfo", 179 },
 	{ "syslog", 116 },
@@ -419,6 +438,7 @@ const struct arch_syscall_def aarch64_syscall_table[] = { \
 	{ "unlinkat", 35 },
 	{ "unshare", 97 },
 	{ "uselib", __PNR_uselib },
+	{ "userfaultfd", 282 },
 	{ "usr26", __PNR_usr26 },
 	{ "usr32", __PNR_usr32 },
 	{ "ustat", __PNR_ustat },
@@ -487,15 +507,15 @@ const char *aarch64_syscall_resolve_num(int num)
 
 
 /**
- * Iterate through the syscall table and return the syscall name
+ * Iterate through the syscall table and return the syscall mapping
  * @param spot the offset into the syscall table
  *
- * Return the syscall name at position @spot or NULL on failure.  This function
- * should only ever be used internally by libseccomp.
+ * Return the syscall mapping at position @spot or NULL on failure.  This
+ * function should only ever be used internally by libseccomp.
  *
  */
-const char *aarch64_syscall_iterate_name(unsigned int spot)
+const struct arch_syscall_def *aarch64_syscall_iterate(unsigned int spot)
 {
 	/* XXX - no safety checks here */
-	return aarch64_syscall_table[spot].name;
+	return &aarch64_syscall_table[spot];
 }
