@@ -2,7 +2,7 @@
  * Enhanced Seccomp x32 Syscall Table
  *
  * Copyright (c) 2013 Red Hat <pmoore@redhat.com>
- * Author: Paul Moore <pmoore@redhat.com>
+ * Author: Paul Moore <paul@paul-moore.com>
  */
 
 /*
@@ -26,7 +26,7 @@
 #include "arch.h"
 #include "arch-x32.h"
 
-/* NOTE: based on Linux 3.19 */
+/* NOTE: based on Linux 4.5-rc4 */
 const struct arch_syscall_def x32_syscall_table[] = { \
 	{ "_llseek", __PNR__llseek },
 	{ "_newselect", __PNR__newselect },
@@ -46,6 +46,7 @@ const struct arch_syscall_def x32_syscall_table[] = { \
 	{ "bind", (X32_SYSCALL_BIT + 49) },
 	{ "bpf", (X32_SYSCALL_BIT + 321) },
 	{ "break", __PNR_break },
+	{ "breakpoint", __PNR_breakpoint },
 	{ "brk", (X32_SYSCALL_BIT + 12) },
 	{ "cachectl", __PNR_cachectl },
 	{ "cacheflush", __PNR_cacheflush },
@@ -64,6 +65,7 @@ const struct arch_syscall_def x32_syscall_table[] = { \
 	{ "clone", (X32_SYSCALL_BIT + 56) },
 	{ "close", (X32_SYSCALL_BIT + 3) },
 	{ "connect", (X32_SYSCALL_BIT + 42) },
+	{ "copy_file_range", (X32_SYSCALL_BIT + 326) },
 	{ "creat", (X32_SYSCALL_BIT + 85) },
 	{ "create_module", __PNR_create_module },
 	{ "delete_module", (X32_SYSCALL_BIT + 176) },
@@ -195,6 +197,7 @@ const struct arch_syscall_def x32_syscall_table[] = { \
 	{ "lstat64", __PNR_lstat64 },
 	{ "madvise", (X32_SYSCALL_BIT + 28) },
 	{ "mbind", (X32_SYSCALL_BIT + 237) },
+	{ "membarrier",  (X32_SYSCALL_BIT + 324) },
 	{ "memfd_create", (X32_SYSCALL_BIT + 319) },
 	{ "migrate_pages", (X32_SYSCALL_BIT + 256) },
 	{ "mincore", (X32_SYSCALL_BIT + 27) },
@@ -203,6 +206,7 @@ const struct arch_syscall_def x32_syscall_table[] = { \
 	{ "mknod", (X32_SYSCALL_BIT + 133) },
 	{ "mknodat", (X32_SYSCALL_BIT + 259) },
 	{ "mlock", (X32_SYSCALL_BIT + 149) },
+	{ "mlock2", (X32_SYSCALL_BIT + 325) },
 	{ "mlockall", (X32_SYSCALL_BIT + 151) },
 	{ "mmap", (X32_SYSCALL_BIT + 9) },
 	{ "mmap2", __PNR_mmap2 },
@@ -223,6 +227,7 @@ const struct arch_syscall_def x32_syscall_table[] = { \
 	{ "msgrcv", (X32_SYSCALL_BIT + 70) },
 	{ "msgsnd", (X32_SYSCALL_BIT + 69) },
 	{ "msync", (X32_SYSCALL_BIT + 26) },
+	{ "multiplexer", __PNR_multiplexer },
 	{ "munlock", (X32_SYSCALL_BIT + 150) },
 	{ "munlockall", (X32_SYSCALL_BIT + 152) },
 	{ "munmap", (X32_SYSCALL_BIT + 11) },
@@ -249,11 +254,15 @@ const struct arch_syscall_def x32_syscall_table[] = { \
 	{ "pipe", (X32_SYSCALL_BIT + 22) },
 	{ "pipe2", (X32_SYSCALL_BIT + 293) },
 	{ "pivot_root", (X32_SYSCALL_BIT + 155) },
+	{ "pkey_alloc", (X32_SYSCALL_BIT + 330) },
+	{ "pkey_free", (X32_SYSCALL_BIT + 331) },
+	{ "pkey_mprotect", (X32_SYSCALL_BIT + 329) },
 	{ "poll", (X32_SYSCALL_BIT + 7) },
 	{ "ppoll", (X32_SYSCALL_BIT + 271) },
 	{ "prctl", (X32_SYSCALL_BIT + 157) },
 	{ "pread64", (X32_SYSCALL_BIT + 17) },
 	{ "preadv", (X32_SYSCALL_BIT + 534) },
+	{ "preadv2", (X32_SYSCALL_BIT + 546) },
 	{ "prlimit64", (X32_SYSCALL_BIT + 302) },
 	{ "process_vm_readv", (X32_SYSCALL_BIT + 539) },
 	{ "process_vm_writev", (X32_SYSCALL_BIT + 540) },
@@ -264,6 +273,7 @@ const struct arch_syscall_def x32_syscall_table[] = { \
 	{ "putpmsg", (X32_SYSCALL_BIT + 182) },
 	{ "pwrite64", (X32_SYSCALL_BIT + 18) },
 	{ "pwritev", (X32_SYSCALL_BIT + 535) },
+	{ "pwritev2", (X32_SYSCALL_BIT + 547) },
 	{ "query_module", __PNR_query_module },
 	{ "quotactl", (X32_SYSCALL_BIT + 179) },
 	{ "read", (X32_SYSCALL_BIT + 0) },
@@ -293,6 +303,10 @@ const struct arch_syscall_def x32_syscall_table[] = { \
 	{ "rt_sigsuspend", (X32_SYSCALL_BIT + 130) },
 	{ "rt_sigtimedwait", (X32_SYSCALL_BIT + 523) },
 	{ "rt_tgsigqueueinfo", (X32_SYSCALL_BIT + 536) },
+	{ "rtas", __PNR_rtas },
+	{ "s390_pci_mmio_read", __PNR_s390_pci_mmio_read },
+	{ "s390_pci_mmio_write", __PNR_s390_pci_mmio_write },
+	{ "s390_runtime_instr", __PNR_s390_runtime_instr },
 	{ "sched_get_priority_max", (X32_SYSCALL_BIT + 146) },
 	{ "sched_get_priority_min", (X32_SYSCALL_BIT + 147) },
 	{ "sched_getaffinity", (X32_SYSCALL_BIT + 204) },
@@ -322,6 +336,7 @@ const struct arch_syscall_def x32_syscall_table[] = { \
 	{ "set_robust_list", (X32_SYSCALL_BIT + 530) },
 	{ "set_thread_area", __PNR_set_thread_area },
 	{ "set_tid_address", (X32_SYSCALL_BIT + 218) },
+	{ "set_tls", __PNR_set_tls },
 	{ "setdomainname", (X32_SYSCALL_BIT + 171) },
 	{ "setfsgid", (X32_SYSCALL_BIT + 123) },
 	{ "setfsgid32", __PNR_setfsgid32 },
@@ -370,6 +385,8 @@ const struct arch_syscall_def x32_syscall_table[] = { \
 	{ "socketcall", __PNR_socketcall },
 	{ "socketpair", (X32_SYSCALL_BIT + 53) },
 	{ "splice", (X32_SYSCALL_BIT + 275) },
+	{ "spu_create", __PNR_spu_create },
+	{ "spu_run", __PNR_spu_run },
 	{ "ssetmask", __PNR_ssetmask },
 	{ "stat", (X32_SYSCALL_BIT + 4) },
 	{ "stat64", __PNR_stat64 },
@@ -377,8 +394,11 @@ const struct arch_syscall_def x32_syscall_table[] = { \
 	{ "statfs64", __PNR_statfs64 },
 	{ "stime", __PNR_stime },
 	{ "stty", __PNR_stty },
+	{ "subpage_prot", __PNR_subpage_prot },
+	{ "swapcontext", __PNR_swapcontext },
 	{ "swapoff", (X32_SYSCALL_BIT + 168) },
 	{ "swapon", (X32_SYSCALL_BIT + 167) },
+	{ "switch_endian", __PNR_switch_endian },
 	{ "symlink", (X32_SYSCALL_BIT + 88) },
 	{ "symlinkat", (X32_SYSCALL_BIT + 266) },
 	{ "sync", (X32_SYSCALL_BIT + 162) },
@@ -386,6 +406,7 @@ const struct arch_syscall_def x32_syscall_table[] = { \
 	{ "sync_file_range2", __PNR_sync_file_range2 },
 	{ "syncfs", (X32_SYSCALL_BIT + 306) },
 	{ "syscall", __PNR_syscall },
+	{ "sys_debug_setcontext", __PNR_sys_debug_setcontext },
 	{ "sysfs", (X32_SYSCALL_BIT + 139) },
 	{ "sysinfo", (X32_SYSCALL_BIT + 99) },
 	{ "syslog", (X32_SYSCALL_BIT + 103) },
@@ -417,6 +438,9 @@ const struct arch_syscall_def x32_syscall_table[] = { \
 	{ "unlinkat", (X32_SYSCALL_BIT + 263) },
 	{ "unshare", (X32_SYSCALL_BIT + 272) },
 	{ "uselib", __PNR_uselib },
+	{ "userfaultfd", (X32_SYSCALL_BIT + 323) },
+	{ "usr26", __PNR_usr26 },
+	{ "usr32", __PNR_usr32 },
 	{ "ustat", (X32_SYSCALL_BIT + 136) },
 	{ "utime", (X32_SYSCALL_BIT + 132) },
 	{ "utimensat", (X32_SYSCALL_BIT + 280) },
@@ -482,15 +506,15 @@ const char *x32_syscall_resolve_num(int num)
 }
 
 /**
- * Iterate through the syscall table and return the syscall name
+ * Iterate through the syscall table and return the syscall mapping
  * @param spot the offset into the syscall table
  *
- * Return the syscall name at position @spot or NULL on failure.  This function
- * should only ever be used internally by libseccomp.
+ * Return the syscall mapping at position @spot or NULL on failure.  This
+ * function should only ever be used internally by libseccomp.
  *
  */
-const char *x32_syscall_iterate_name(unsigned int spot)
+const struct arch_syscall_def *x32_syscall_iterate(unsigned int spot)
 {
 	/* XXX - no safety checks here */
-	return x32_syscall_table[spot].name;
+	return &x32_syscall_table[spot];
 }
