@@ -4,7 +4,7 @@
 # Seccomp Library test program
 #
 # Copyright (c) 2013 Red Hat <pmoore@redhat.com>
-# Author: Paul Moore <pmoore@redhat.com>
+# Author: Paul Moore <paul@paul-moore.com>
 #
 
 #
@@ -35,7 +35,7 @@ def test():
         quit(1)
     util.install_trap()
 
-    fd = os.open("/dev/null", os.O_WRONLY|os.O_CREAT, 0600)
+    fd = os.open("/dev/null", os.O_WRONLY|os.O_CREAT)
 
     f = SyscallFilter(TRAP)
     # NOTE: additional syscalls required for python
@@ -44,10 +44,11 @@ def test():
     f.add_rule(ALLOW, "rt_sigaction")
     f.add_rule(ALLOW, "rt_sigreturn")
     f.add_rule(ALLOW, "exit_group")
+    f.add_rule(ALLOW, "brk")
     f.load()
 
     try:
-        if not os.write(fd, "testing") == len("testing"):
+        if not os.write(fd, b"testing") == len("testing"):
             raise IOError("failed to write the full test string")
         quit(160)
     except OSError as ex:
